/***************************************************************************
                     configdialog.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "configdialog.h"
#include "accountmanager.h"
#include "notifiermanager.h"
#include "configgeneralpage.h"
#include "configpluginspage.h"

#include <qwidget.h>
#include <qvbox.h>

#include <kglobal.h>
#include <klocale.h>
#include <kstddirs.h>
#include <kiconloader.h>
#include <kicontheme.h>
#include <kapp.h>
#include <kconfig.h>

ConfigDialog::ConfigDialog(AccountManager *accountMgr,
			   NotifierManager *notifierMgr,
			   QWidget *parent,
			   const char *name)
  : KDialogBase(KDialogBase::IconList, i18n("Configuration"),
		KDialogBase::Ok | KDialogBase::Apply | KDialogBase::Cancel,
		KDialogBase::Ok, parent, name), _configPageList(), _modified(false)
  {
  initPages(accountMgr, notifierMgr);
 
  KConfig *config = kapp->config();
  readOptions(*config);

  // The apply button will be enabled when something is modified
  enableButton(KDialogBase::Apply, false);
  }

ConfigDialog::~ConfigDialog()
  {
  }

void ConfigDialog::initPages(AccountManager *accountMgr, NotifierManager *notifierMgr)
  {
  QPixmap icon;
  QVBox *page;
  ConfigPage *configPage;
  
  icon = KGlobal::iconLoader()->loadIcon("gear",
 						 KIcon::Desktop);
  page = addVBoxPage(i18n("General"), i18n("General"), icon);
  configPage = new ConfigGeneralPage(page, "configGeneralPage");
  _configPageList.append(configPage);
  connect(configPage, SIGNAL(modified()), this, SLOT(slotModified()));
  
  icon = KGlobal::iconLoader()->loadIcon("identity",
					 KIcon::Desktop);
  page = addVBoxPage(i18n("Accounts"), i18n("Accounts"), icon);
  configPage = new ConfigPluginsPage(accountMgr, page, "configAccountsPage");
  _configPageList.append(configPage);
  connect(configPage, SIGNAL(modified()), this, SLOT(slotModified()));
 
 
  icon = KGlobal::iconLoader()->loadIcon("sound",
					 KIcon::Desktop);
  page = addVBoxPage(i18n("Notifiers"), i18n("Notifiers"), icon);
  configPage = new ConfigPluginsPage(notifierMgr, page, "configNotifiersPage");
  _configPageList.append(configPage);
  connect(configPage, SIGNAL(modified()), this, SLOT(slotModified()));
 
 
//   icon = KGlobal::iconLoader()->loadIcon("viewmag",
// 					 KIcon::Desktop);
//   page = addVBoxPage(i18n("Message Filters"),
// 		     i18n("Message Filters"), icon);
//   configPage = new ConfigPluginsPage(filterMgr, page, "configFiltersPage");
//   _configPageList.append(configPage);
//   connect(configPage, SIGNAL(modified()), this, SLOT(slotModified()));
  }

void ConfigDialog::readOptions(KConfig &config)
  {
  QListIterator<ConfigPage> iter(_configPageList);
  for (iter.toFirst(); iter.current(); ++iter)
    iter.current()->readOptions(config);
  }

void ConfigDialog::writeOptions(KConfig &config)
  {
  QListIterator<ConfigPage> iter(_configPageList);
  for (iter.toFirst(); iter.current(); ++iter)
    iter.current()->writeOptions(config);
  }

/////////////////////////////
// SLOTS

void ConfigDialog::slotOk()
  {
  if (_modified)
    slotApply();
  
  KDialogBase::slotOk();
  }

void ConfigDialog::slotApply()
  {
  KConfig *config = kapp->config();
  
  writeOptions(*config);

  _modified = false;
  enableButton(KDialogBase::Apply, false);
  }

void ConfigDialog::slotModified()
  {
  enableButton(KDialogBase::Apply, true);
  _modified = true;
  }
