/***************************************************************************
                       configgeneralpage.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qspinbox.h>
#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qlabel.h>

#include <kconfig.h>
#include <kglobal.h>
#include <klocale.h>
#include <kstddirs.h>

#include "configgeneralpage.h"

ConfigGeneralPage::ConfigGeneralPage(QWidget *parent, const char *name)
      : ConfigPage(parent, name), _checkAtStartup(NULL),
	_startMinimized(NULL), _intervalTime(NULL)
  {
  QGroupBox *startupBox = new QGroupBox(2, Qt::Vertical, i18n("Startup"),
					this, "startupBox");

  _checkAtStartup = new QCheckBox(i18n("Check mail at startup"),
				      startupBox, "checkAtStartup");
  _checkAtStartup->setMinimumSize(_checkAtStartup->sizeHint());
  connect(_checkAtStartup, SIGNAL(toggled(bool)),
	  this, SLOT(slotModified(bool)));
  
  _startMinimized = new QCheckBox(i18n("Start minimized"),
				       startupBox, "startMinimized");
  _startMinimized->setMinimumSize(_startMinimized->sizeHint());
  connect(_startMinimized, SIGNAL(toggled(bool)),
	  this, SLOT(slotModified(bool)));

  QGroupBox *intervalBox = new QGroupBox(2, Qt::Horizontal, i18n("Interval"),
					 this, "intervalBox");
  
  (void) new QLabel(i18n("Interval between checks:"), intervalBox,
			     "label");

  _intervalTime = new QSpinBox(1, 1000, 1, intervalBox, "intervalTime");
  _intervalTime->setSuffix(i18n("  minutes"));
  _intervalTime->setMinimumSize(_intervalTime->sizeHint());
  connect(_intervalTime, SIGNAL(valueChanged(int)),
	  this, SLOT(slotModified(int)));
  
  QWidget *spacer = new QWidget(this, "spacer");
  setStretchFactor(spacer, 1);
  }

ConfigGeneralPage::~ConfigGeneralPage()
  {
  }

void ConfigGeneralPage::readOptions(KConfig &config)
  {
  config.setGroup("General");
  
  _startMinimized->setChecked(config.readBoolEntry("StartMinimized", false));
  _checkAtStartup->setChecked(config.readBoolEntry("CheckAtStartup", false));
  _intervalTime->setValue(config.readNumEntry("IntervalTime", 5));
  }

void ConfigGeneralPage::writeOptions(KConfig &config)
  {
  config.setGroup("General");
  
  config.writeEntry("StartMinimized", _startMinimized->isChecked());
  config.writeEntry("CheckAtStartup", _checkAtStartup->isChecked());
  config.writeEntry("IntervalTime", _intervalTime->value());
  }

