/***************************************************************************
                       configpluginspage.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlistview.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qlabel.h>

#include <kconfig.h>
#include <kbuttonbox.h>
#include <klocale.h>
#include <kdebug.h>

#include "configpluginspage.h"
#include "pluginmanager.h"

ConfigPluginsPage::ConfigPluginsPage(PluginManager *pluginMgr,
					 QWidget *parent, const char *name)
      : ConfigPage(parent, name), _pluginListView(NULL), _addButton(NULL),
	_removeButton(NULL), _modifiyButton(NULL),
	_pluginList(), _pluginMgr(pluginMgr)
  {
  QGroupBox *availBox = new QGroupBox(2, Qt::Horizontal,
				      i18n("Installed Plugins"), this,
				      "availBox");
  
  _pluginListView = new QListView(availBox,
				  "pluginsListView");
  _pluginListView->addColumn(i18n("Name"));
  _pluginListView->addColumn(i18n("Type"));
  _pluginListView->setAllColumnsShowFocus(true);
 
  KButtonBox *buttonBox = new KButtonBox(availBox, Qt::Vertical);
  _addButton = buttonBox->addButton(i18n("&Add"));
  _removeButton = buttonBox->addButton(i18n("&Remove"));
  _modifiyButton = buttonBox->addButton(i18n("&Modify"));
 
  _removeButton->setEnabled(false);
  _modifiyButton->setEnabled(false);

    // Connect the slots
    connect(_pluginListView, SIGNAL(currentChanged(QListViewItem *)),
	    this, SLOT(slotListViewCurrentChanged(QListViewItem *)));
    connect(_addButton, SIGNAL(clicked()),
	    this, SLOT(slotAddButtonClicked()));
    connect(_removeButton, SIGNAL(clicked()),
	    this, SLOT(slotRemoveButtonClicked()));
    connect(_modifiyButton, SIGNAL(clicked()),
	    this, SLOT(slotModifyButtonClicked()));
  }

ConfigPluginsPage::~ConfigPluginsPage()
  {
  }

void ConfigPluginsPage::readOptions(KConfig &config)
  {
  KNMPlugin *plugin = NULL;
  
  // Load the plugin list, then for each one, build a qlistviewitem
  _pluginMgr->readPlugins(config, _pluginList);
  
  QListIterator<KNMPlugin> iter(_pluginList);
  for (iter.toFirst(); iter.current(); ++iter)
    {
      plugin = *iter;
      
      (void) new QListViewItem(_pluginListView, plugin->getPluginName(),
   			       plugin->getPluginInfo().getPluginType(),
   			       QString::number(plugin->getUniqueID()));
    }
  }

void ConfigPluginsPage::writeOptions(KConfig &config)
  {
  // Tell the plugin manager to write out list of plugins, then reread them
  _pluginMgr->writePlugins(config, _pluginList);
  _pluginMgr->readOptions(config);
  }

//////////////////////////////////
// SLOTS
void ConfigPluginsPage::slotAddButtonClicked()
  {
  AddPluginDialog dialog(_pluginMgr,
			  this, "addPluginDialog");

  if (dialog.exec())
    {
      QString selected = dialog.getSelectedPlugin();
      if (selected != QString::null)
	{
	  KNMPlugin *plugin = _pluginMgr->createPlugin(selected);
	  _pluginList.append(plugin);

	  QListViewItem *item =new QListViewItem(_pluginListView,
						 plugin->getPluginName(),
						 plugin->getPluginInfo().getPluginType(),
						 QString::number(plugin->getUniqueID()));

	  // now select the item we just added and call slotModify
	  _pluginListView->setSelected(item, true);

	  emit modified();
	  slotModifyButtonClicked();
	}
    }
  }

void ConfigPluginsPage::slotRemoveButtonClicked()
  {
    QListViewItem *item = _pluginListView->currentItem();

  if (item == NULL)
    return;
  
  Q_UINT32 id = item->text(2).toInt();
  
  KNMPlugin *plugin = PluginManager::findPlugin(id, _pluginList);
  if (plugin)
    {
      delete item;
      _pluginList.removeRef(plugin);
      emit modified();
    }
  }

void ConfigPluginsPage::slotModifyButtonClicked()
  {
  QListViewItem *item = _pluginListView->currentItem();

  if (item == NULL)
    return;
  
  Q_UINT32 id = item->text(2).toInt();
  
  KNMPlugin *plugin = PluginManager::findPlugin(id, _pluginList);
  if (plugin)
    {
      if (plugin->execConfigDialog(this, "pluginConfigDialog"))
	{
	  emit modified();
	  item->setText(0, plugin->getPluginName());
	}
    }
  }

void ConfigPluginsPage::slotListViewCurrentChanged(QListViewItem *item)
  {
  _modifiyButton->setEnabled(item != NULL);
  _removeButton->setEnabled(item != NULL);
  }

/////////////////////////////////
// AddPluginDialog

AddPluginDialog::AddPluginDialog(PluginManager *pluginMgr,
				   QWidget *parent, const char *name)
      : KDialogBase(KDialogBase::Plain, i18n("Plugin Type"),
		    KDialogBase::Ok | KDialogBase::Cancel, KDialogBase::Ok,
		    parent, name),
	_buttonGroup(NULL)
  {
  QRadioButton *button;
  QLabel *label;
  
  _buttonGroup = new QButtonGroup(2, Qt::Horizontal, i18n("New Plugin Type"),
				  plainPage(), "buttonGroup");

  QStringList pluginTypes = pluginMgr->getPluginList();
  QStringList::ConstIterator iter;
  for (iter = pluginTypes.begin(); iter != pluginTypes.end(); ++iter)
    {
      button =  new QRadioButton(i18n(*iter), _buttonGroup,
				 "pluginTypeRadioButton");    
      button->setMinimumSize(button->sizeHint());

      KNMPluginInfo *info = pluginMgr->createPluginInfo(*iter);
      QString infoString = info->getDescription() + "\n";
      infoString += info->getAuthor() + "   v";
      infoString += info->getVersion();
      
      label = new QLabel(i18n(infoString), _buttonGroup, "label");
      label->setMinimumSize(label->sizeHint());
      delete info;
    }

  _buttonGroup->setMinimumSize(_buttonGroup->sizeHint());
  _buttonGroup->setButton(0);

  plainPage()->setMinimumSize(_buttonGroup->sizeHint());
  }

QString AddPluginDialog::getSelectedPlugin()
  {
  QString text = QString::null;
  QButton *button = _buttonGroup->selected();

  if (button != NULL)
    text = button->text();

  return text;
  }
