/***************************************************************************
                          knewmail.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qtimer.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kdebug.h>
#include <kmessagebox.h>

// application specific includes
#include "knewmail.h"
#include "accountmanager.h"
#include "notifiermanager.h"
#include "configdialog.h"

#define ID_STATUS_MSG        1
#define ID_STATUS_LAST_CHECK 2
#define ID_STATUS_NUM_MAILS  3

KNewMailApp::KNewMailApp(QWidget* , const char* name)
      : KMainWindow(0, name)
{
  _config=kapp->config();

  // set as main widget
  kapp->setMainWidget(this);

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initStatusBar();
  initActions();
  initManagers();
 
  ///////////////////////////////////////////////////////////////////
  // set actions at startup
  _fileEnabled->setEnabled(true);
  _fileEnabled->setChecked(true);
  _fileCheck->setEnabled(true);
  _fileCancel->setEnabled(false);

  readOptions();

  setCaption("KNewMail",false);

  if (_config->readBoolEntry("StartMinimized", false))
    showMinimized();
  else
    show();

  // setup the interval timer
  Q_UINT32 interval = _config->readNumEntry("IntervalTime", 5);
  _intervalTimer = new QTimer(this, "intervalTimer");
  connect(_intervalTimer, SIGNAL(timeout()),
	  this, SLOT(slotCheckTimerTimeout()));
  _intervalTimer->start(interval * 60 * 1000);

  if (_config->readBoolEntry("CheckAtStartup", false))
    {
    QTimer *startupTimer = new QTimer(this, "startupTimer");
    connect(startupTimer, SIGNAL(timeout()),
	    this, SLOT(slotCheckTimerTimeout()));
    // wait 1 second then run
    startupTimer->start(1 * 1000, true);
    }
}

KNewMailApp::~KNewMailApp()
  {
  disconnect(_accountMgr, SIGNAL(doneChecking(QList<KNMMessage> &)),
	     this, SLOT(slotDoneChecking(QList<KNMMessage> &)));
  disconnect(_accountMgr,
	     SIGNAL(errorOccured(const QString &, const QString &)),
	     this,
	     SLOT(slotAccountErrorOccured(const QString &, const QString &))); 
  delete _accountMgr;
  _accountMgr = NULL;

  disconnect(_notifierMgr, SIGNAL(forceCheck()), this, SLOT(slotFileCheck()));
  disconnect(_notifierMgr, SIGNAL(markMessageForDelete(Q_UINT32)),
	 this, SLOT(slotMarkMessageForDelete(Q_UINT32)));
  disconnect(_notifierMgr,
	    SIGNAL(errorOccured(const QString &, const QString &)),
	 this,
	 SLOT(slotAccountErrorOccured(const QString &, const QString &)));
  delete _notifierMgr;
  _notifierMgr = NULL;
  }

void KNewMailApp::initActions()
{

  _fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()),
			      actionCollection());

  _fileCheck = new KAction(i18n("&Check Mail"), "mail_get",
			   KAccel::stringToKey("Ctrl+C"),
			   this, SLOT(slotFileCheck()), actionCollection(),
			   "file_check");
  
  _fileCancel = new KAction(i18n("C&ancel Check"), "stop",
			   KAccel::stringToKey("Ctrl+A"),
			   this, SLOT(slotFileCancel()), actionCollection(),
			   "file_cancel");
  
  _fileEnabled = new KToggleAction(i18n("&Enable Checking"), "gear",
				   KAccel::stringToKey("Ctrl+E"),
				   this, SLOT(slotFileEnabled()),
				   actionCollection(),
				   "file_enabled");

  _settingsConfigure = new KAction(i18n("&Configure KNewMail"), "configure",
				   0,
				   this, SLOT(slotSettingsConfigure()),
				   actionCollection(),
				   "settings_configure");
  
  _viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()),
					actionCollection());
  _viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()),
					    actionCollection());

  _fileQuit->setStatusText(i18n("Quits the application"));
  _viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
  _viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));

  // use the absolute path to your knewmailui.rc file for testing
  // purpose in createGUI();
  createGUI();
}


void KNewMailApp::initStatusBar()
{
  statusBar()->insertItem(i18n("Ready"), ID_STATUS_MSG);
  statusBar()->insertItem(i18n("Last Check: 00:00:00"), ID_STATUS_LAST_CHECK,
			  0, true);
  statusBar()->insertItem(i18n("Num Mails: 0"), ID_STATUS_NUM_MAILS,
			  0, true);
}

void KNewMailApp::initManagers()
{
 _accountMgr = new AccountManager(this, "accountManager");
 connect(_accountMgr, SIGNAL(doneChecking(QList<KNMMessage> &)),
	 this, SLOT(slotDoneChecking(QList<KNMMessage> &)));
 connect(_accountMgr, SIGNAL(errorOccured(const QString &, const QString &)),
	 this,
	 SLOT(slotAccountErrorOccured(const QString &, const QString &)));

 _accountMgr->readOptions(*_config);
 
 // Create the notifier
 _notifierMgr = new NotifierManager(this, "notifierManager");
 connect(_notifierMgr, SIGNAL(forceCheck()), this, SLOT(slotFileCheck()));
 connect(_notifierMgr, SIGNAL(markMessageForDelete(Q_UINT32)),
	 this, SLOT(slotMarkMessageForDelete(Q_UINT32)));
 connect(_notifierMgr, SIGNAL(unmarkMessageForDelete(Q_UINT32)),
	 this, SLOT(slotUnmarkMessageForDelete(Q_UINT32)));
 connect(_notifierMgr, SIGNAL(errorOccured(const QString &, const QString &)),
	 this,
	 SLOT(slotAccountErrorOccured(const QString &, const QString &)));

 _notifierMgr->readOptions(*_config);
}

void KNewMailApp::saveOptions()
{	
  _config->setGroup("GUI");
  _config->writeEntry("Geometry", size());
  _config->writeEntry("Show Toolbar", _viewToolBar->isChecked());
  _config->writeEntry("Show Statusbar",_viewStatusBar->isChecked());
  _config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());

    // State of the buttons
  _config->setGroup("General");
  _config->writeEntry("Enabled", _fileEnabled->isChecked());
}


void KNewMailApp::readOptions()
{
  _config->setGroup("GUI");

  // bar status settings
  bool bViewToolbar = _config->readBoolEntry("Show Toolbar", true);
  _viewToolBar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = _config->readBoolEntry("Show Statusbar", true);
  _viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();


  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos = (KToolBar::BarPosition) _config->readNumEntry("ToolBarPos",
							     KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);
	
  QSize size = _config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }

  // State of the buttons
  _config->setGroup("General");

  bool enabled = _config->readBoolEntry("Enabled", true);
  _fileEnabled->setChecked(enabled);
}

void KNewMailApp::saveProperties(KConfig *)
{

}


void KNewMailApp::readProperties(KConfig*)
{

}		

bool KNewMailApp::queryClose()
{
 return true;
}

bool KNewMailApp::queryExit()
{
  saveOptions();
  return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KNewMailApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
  KMainWindow* w;
  if(memberList)
  {
    for(w=memberList->first(); w!=0; w=memberList->first())
    {
      // only close the window if the closeEvent is accepted. If the
      // user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
      if(!w->close())
	break;
    }
  }	
  slotStatusMsg(i18n("Ready"));
}

void KNewMailApp::slotFileCheck()
  {
  if (!_accountMgr->isChecking())
    {
      // disable the check button
      _fileCheck->setEnabled(false);
      _fileCancel->setEnabled(true);

      // Update the status message
      slotStatusMsg(i18n("Checking Messages..."));
      
      // Tell the account manager to check messages
      _accountMgr->checkMessages();
    }
  }

void KNewMailApp::slotFileCancel()
  {
  if (_accountMgr->isChecking())
    {
      slotStatusMsg(i18n("Canceling Check..."));
      _accountMgr->cancelCheck();
    }
  }

void KNewMailApp::slotFileEnabled()
  {
  }

void KNewMailApp::slotSettingsConfigure()
  {
  ConfigDialog dialog(_accountMgr, _notifierMgr,
		      this, "configDialog");

  dialog.exec();
  
  // restart the timer
  _intervalTimer->stop();
  
  Q_UINT32 interval = _config->readNumEntry("IntervalTime", 5);
  _intervalTimer->start(interval * 60 * 1000);
  }

void KNewMailApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggling toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!_viewToolBar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }		

  slotStatusMsg(i18n("Ready"));
}

void KNewMailApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!_viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready"));
}


void KNewMailApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}

void KNewMailApp::slotDoneChecking(QList<KNMMessage> &msgs)
  {
  _notifierMgr->updateState(msgs);

  // Update the GUI
  _fileCancel->setEnabled(false);
  _fileCheck->setEnabled(true);

  slotStatusMsg(i18n("Ready"));

  QString newMsg = "Last Check: ";
  newMsg += QTime::currentTime().toString();
  statusBar()->changeItem(newMsg, ID_STATUS_LAST_CHECK);

  newMsg = "Num Mails: ";
  newMsg += QString::number(msgs.count());
  statusBar()->changeItem(newMsg, ID_STATUS_NUM_MAILS);
  }

void KNewMailApp::slotAccountErrorOccured(const QString &accountName,
			     const QString &errorDesc)
  {
  QString message;

  message = "An error occur with account " + accountName + ". \n";
  message += "The server said: \n" + errorDesc;
  
  KMessageBox::error(this, message, "KNewMail Error", false);
  }

void KNewMailApp::slotMarkMessageForDelete(Q_UINT32 msgID)
  {
  _notifierMgr->updateMarkMessageForDelete(msgID);
  _accountMgr->markMessageForDelete(msgID);
  }

void KNewMailApp::slotUnmarkMessageForDelete(Q_UINT32 msgID)
  {
  _notifierMgr->updateUnmarkMessageForDelete(msgID);
  _accountMgr->unmarkMessageForDelete(msgID);
  }

void KNewMailApp::slotCheckTimerTimeout()
  {
  if (_fileEnabled->isChecked())
    slotFileCheck();
  }
