/***************************************************************************
                          knewmail.h  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KNEWMAIL_H
#define KNEWMAIL_H
 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "knmmessage.h"

// include files for Qt

// include files for KDE 
#include <kapp.h>
#include <kmainwindow.h>
#include <kaccel.h>
#include <kaction.h>

class QTimer;
class AccountManager;
class NotifierManager;

/**
  * The base class for KNewMail application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
  * and statusbar. An instance of KNewMailView creates your center view,
  * which is connected to the window's Doc object.
  * KNewMailApp reimplements the methods that KMainWindow provides for
  * main window handling and supports full session management as well
  * as using KActions.
  *
  * @see KMainWindow
  * @see KApplication
  * @see KConfig
  *
  * @author Mike Pilone
  */
class KNewMailApp : public KMainWindow
{
  Q_OBJECT

  friend class KNewMailView;

  public:
    /** construtor of KNewMailApp, calls all init functions to create the
     * application.
     */
    KNewMailApp(QWidget* parent=0, const char* name=0);
    ~KNewMailApp();

  protected:
    /** save general Options like all bar positions and status as well as
     * the geometry and the recent file list to the configuration file
     */ 	
    void saveOptions();
  
    /** read general Options again and initialize all variables like the
     * recent file list
     */
    void readOptions();
  
    /** initializes the KActions of the application
     */
    void initActions();
  
    /** sets up the statusbar for the main window by initialzing a
     * statuslabel.
     */
    void initStatusBar();
  
    void initManagers();

    /** queryClose is called by KTMainWindow on each closeEvent of a
     * window. Against the default implementation (only returns true),
     * this calles saveModified() on the document object to ask if the
     * document shall be saved if Modified; on cancel the closeEvent
     * is rejected.
     *
     * @see KTMainWindow#queryClose
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryClose();
  
    /** queryExit is called by KTMainWindow when the last window
     * of the application is going to be closed during the closeEvent().
     * Against the default implementation that just returns true, this
     * calls saveOptions() to save the settings of the last window's	
     * properties.
     *
     * @see KTMainWindow#queryExit
     * @see KTMainWindow#closeEvent
     */
    virtual bool queryExit();
  
    /** saves the window properties for each open window during session
     * end to the session config file, including saving the currently
     * opened file by a temporary filename provided by KApplication.
     *
     * @see KTMainWindow#saveProperties
     */
    virtual void saveProperties(KConfig *cfg);
  
    /** reads the session config file and restores the application's
     * state including the last opened files and documents by reading the
     * temporary files saved by saveProperties()
     *
     * @see KTMainWindow#readProperties
     */
    virtual void readProperties(KConfig *cfg);

  public slots:

  
    /** closes all open windows by calling close() on each memberList
     * item until the list is empty, then quits the application.
     * If queryClose() returns false because the user canceled the
     * saveModified() dialog, the closing breaks.
     */
    void slotFileQuit();

  void slotFileCheck();
  void slotFileCancel();
  void slotFileEnabled();

  
  void slotSettingsConfigure();
  
    /** toggles the toolbar
     */
    void slotViewToolBar();
  
    /** toggles the statusbar
     */
    void slotViewStatusBar();
  
    /** changes the statusbar contents for the standard label permanently,
     * used to indicate current actions.
     * 
     * @param text the text that is displayed in the statusbar
     */
    void slotStatusMsg(const QString &text);

void slotDoneChecking(QList<KNMMessage> &msgs);
void slotAccountErrorOccured(const QString &accountName,
			     const QString &errorDesc);

void slotMarkMessageForDelete(Q_UINT32 msgID);
void slotUnmarkMessageForDelete(Q_UINT32 msgID);

void slotCheckTimerTimeout();

  private:
    /** the configuration object of the application */
    KConfig *_config;
 
    AccountManager  *_accountMgr;
    NotifierManager *_notifierMgr;

    // KAction pointers to enable/disable actions
    KAction* _fileQuit;
  KAction *_fileCheck;
  KAction *_fileCancel;
  KToggleAction *_fileEnabled;
  
  KAction *_settingsConfigure;
  
    KToggleAction* _viewToolBar;
    KToggleAction* _viewStatusBar;

QTimer *_intervalTimer;
};
 
#endif // KNEWMAIL_H
