/***************************************************************************
                     accountmanager.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "notifiermanager.h"
#include "knmnotifier.h"

NotifierManager::NotifierManager(QObject *parent, const char *name)
  : PluginManager(parent, name)
  {
  loadPlugins();
  }

NotifierManager::~NotifierManager()
  {
  }

void NotifierManager::initPlugin(KNMPlugin *plugin)
  {
  KNMNotifier *notifier = (KNMNotifier*)plugin;

  connect(notifier, SIGNAL(forceCheck()),
	  this, SLOT(slotForceCheck()));
  connect(notifier, SIGNAL(markMessageForDelete(Q_UINT32)),
	  this, SLOT(slotMarkMessageForDelete(Q_UINT32)));
  connect(notifier, SIGNAL(unmarkMessageForDelete(Q_UINT32)),
	  this, SLOT(slotUnmarkMessageForDelete(Q_UINT32)));
  
  notifier->initGUI();
  }

void NotifierManager::updateState(QList<KNMMessage> &msgs)
  {
  // Run through all notifiers and update them
  QListIterator<KNMPlugin> iter(_pluginList);

  for (iter.toFirst(); iter.current(); ++iter)
    {
    ((KNMNotifier*)iter.current())->updateState(msgs);
    }
  }

void NotifierManager::updateMarkMessageForDelete(Q_UINT32 msgID)
  {
   // Run through all notifiers and update them
  QListIterator<KNMPlugin> iter(_pluginList);

  for (iter.toFirst(); iter.current(); ++iter)
    {
    ((KNMNotifier*)iter.current())->updateMarkMessageForDelete(msgID);
    }
  }

void NotifierManager::updateUnmarkMessageForDelete(Q_UINT32 msgID)
  {
  // Run through all notifiers and update them
  QListIterator<KNMPlugin> iter(_pluginList);

  for (iter.toFirst(); iter.current(); ++iter)
    {
    ((KNMNotifier*)iter.current())->updateUnmarkMessageForDelete(msgID);
    }
  }

//////////////////////////////
// SLOTS

void NotifierManager::slotForceCheck()
  {
  emit forceCheck();
  }

void NotifierManager::slotMarkMessageForDelete(Q_UINT32 msgID)
  {
  emit markMessageForDelete(msgID);
  }

void NotifierManager::slotUnmarkMessageForDelete(Q_UINT32 msgID)
  {
  emit unmarkMessageForDelete(msgID);
  }

