#include <kdebug.h>
#include <kglobal.h>
#include <kiconloader.h>

#include "animwidget.h"

AnimWidget::AnimWidget(const QString &animIconName,
		       QWidget *parent,
		       const char *name)
  : QWidget(parent, name), _animateTimer(),
    _pixmapList(), _currentIndex(0)
  {
  QPixmap *pixmap;
  
  QStringList icons = KGlobal::iconLoader()->loadAnimated(animIconName,
							  KIcon::Small);
  
  QStringList::Iterator iter;
  for (iter = icons.begin(); iter != icons.end(); ++iter)
    {
    pixmap = new QPixmap(*iter);
    _pixmapList.append(pixmap);
    }

  connect(&_animateTimer, SIGNAL(timeout()),
	  this, SLOT(slotAnimateTimeout()));

  // Prime the icon
  slotAnimateTimeout();
  }

AnimWidget::~AnimWidget()
  {
  }


void AnimWidget::start()
  {
  _animateTimer.start(100, false);
  }

void AnimWidget::stop()
  {
  _animateTimer.stop();
  }

void AnimWidget::slotAnimateTimeout()
  {
  if (_pixmapList.count() == 0)
    {
    kdDebug() << "AnimWidget::slotAnimateTimeout: No Icons loaded\n";
    _animateTimer.stop();
    return;
    }
  
  QPixmap *pixmap = _pixmapList.at(_currentIndex);

  _currentIndex++;
  if (_currentIndex >= _pixmapList.count())
    _currentIndex = 0;

  if (pixmap != NULL)
    setBackgroundPixmap(*pixmap);
  }
