/***************************************************************************
                 knmlistviewnotifier.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlistview.h>
#include <qpixmap.h>

#include <kmainwindow.h>
#include <kconfig.h>
#include <klistview.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kicontheme.h>
#include <kdebug.h>

#include "knmlistviewnotifier.h"
#include "knmplugin.h"

extern "C" {
  KNMPluginInfo *initListViewPluginInfo(QObject *parent)
    {
    return new KNMPluginInfo("ListView",
			     "A multi-column list view in the main window.",
			     "Mike Pilone <mpilone@slac.com>", "1.0.0",
			     parent, "knmlistviewnotifierinfo");
    }
  
  KNMNotifier *initListViewPlugin(KMainWindow *parent)
    {
    return new KNMListViewNotifier(parent, "knmlistviewnotifier");
    }
}

KNMListViewNotifier::KNMListViewNotifier(KMainWindow *parent,
					 const char *name)
  : KNMNotifier(initListViewPluginInfo(parent),
		parent, name), _listView(NULL), _mainWin(parent)
  {
  }

KNMListViewNotifier::~KNMListViewNotifier()
  {
  delete _listView;
  _listView = NULL;
  }

void KNMListViewNotifier::readOptions(KConfig &)
  {
  }

void KNMListViewNotifier::writeOptions(KConfig &)
  {
  }

void KNMListViewNotifier::updateState(QList<KNMMessage> &msgs)
  {
  KNMMessage *message = NULL;
  QString from("Unknown");
  QCString subject("Unknown");

  if (_listView)
    {
    _listView->clear();

    QListIterator<KNMMessage> iter(msgs);
    for (iter.toFirst(); iter.current(); ++iter)
      {
      message = *iter;

      from = message->firstSender().phrase();
      subject = message->subject().asString();
      
      (void) new QListViewItem(_listView,
			       QString(""),
			       message->getAccountName(),
			       from,
			       subject,
			       QString::number(message->getUniqueID()));

      if (message->isMarkedForDelete())
	updateMarkMessageForDelete(message->getUniqueID());
      }
    }
  }

void KNMListViewNotifier::updateMarkMessageForDelete(Q_UINT32 msgID)
  {
  setItemPixmap(true, msgID);  
  }

void KNMListViewNotifier::updateUnmarkMessageForDelete(Q_UINT32 msgID)
  {
  setItemPixmap(false, msgID);
  }

void KNMListViewNotifier::initGUI()
  {
  delete _listView;
  _listView = NULL;
  
  _listView = new KListView(_mainWin, "listView");
  _mainWin->setCentralWidget(_listView);

  _listView->addColumn(i18n("D"));
  _listView->addColumn(i18n("To"));
  _listView->addColumn(i18n("From"));
  _listView->addColumn(i18n("Subject"));

  _listView->setAllColumnsShowFocus(true);

  // connect to executed signal to apply with kde standards
  connect(_listView, SIGNAL(executed(QListViewItem*)),
	  this, SLOT(slotItemDoubleClicked(QListViewItem*)));
  }

void KNMListViewNotifier::setItemPixmap(bool on, Q_UINT32 msgID)
  {
  QPixmap icon;
  QPixmap empty;
  QListViewItem *item;
  bool found = false;
  
  icon = KGlobal::iconLoader()->loadIcon("fileclose",
					 KIcon::Small);

  // iterate through all the listview items and try to find this one
  QListViewItemIterator iter(_listView);
  for (; iter.current() && !found; ++iter)
    {
    item = iter.current();
    if (item->text(4).toUInt() == msgID)
      {
      found = true;
      if (on)
	item->setPixmap(0, icon);
      else
	{
	item->setPixmap(0, empty);
	}
      }
    }
  
  }
///////////////////////////////////////////
// SLOTS
void KNMListViewNotifier::slotItemDoubleClicked(QListViewItem *item)
  {
  if (item)
    {
    if (item->pixmap(0) != NULL)
      {
      // undelete
      emit unmarkMessageForDelete(item->text(4).toUInt());
      }
    else
      {
      emit markMessageForDelete(item->text(4).toUInt());
      }
    }
  }
