/***************************************************************************
                         knmplugin.h  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlabel.h>
#include <qtooltip.h>
#include <qfont.h>

#include <kconfig.h>
#include <kmainwindow.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kwin.h>

#include "knmnumberdocknotifier.h"

extern "C" {
  KNMPluginInfo *initNumberDockPluginInfo(QObject *parent)
    {
    return new KNMPluginInfo("NumberDock",
			     "Displays a small LCD style number docked in the panel.",
			     "Mike Pilone <mpilone@slac.com>", "1.0.0",
			     parent, "knmnumberdocknotifierinfo");
    }
  
  KNMNotifier *initNumberDockPlugin(KMainWindow *parent)
    {
    return new KNMNumberDockNotifier(parent, "knmnumberdocknotifier");
    }
}

KNMNumberDockNotifier::KNMNumberDockNotifier(KMainWindow *parent, const char *name)
  : KNMNotifier(initNumberDockPluginInfo(parent), parent, name),
    _panelNumber(NULL)
  {
  }

KNMNumberDockNotifier::~KNMNumberDockNotifier()
  {
  if (_panelNumber)
    delete _panelNumber;
  }

void KNMNumberDockNotifier::readOptions(KConfig &config)
  {
  KNMNotifier::readOptions(config);
  }

void KNMNumberDockNotifier::writeOptions(KConfig &config)
  {
  KNMNotifier::writeOptions(config);
  }

void KNMNumberDockNotifier::updateState(QList<KNMMessage> &msgs)
  {
  if (_panelNumber)
    _panelNumber->setText(QString::number(msgs.count()));
  }

void KNMNumberDockNotifier::updateMarkMessageForDelete(Q_UINT32)
  {
  // no-op
  }

void KNMNumberDockNotifier::updateUnmarkMessageForDelete(Q_UINT32)
  {
  // no-op
  }

void KNMNumberDockNotifier::initGUI()
  {
  _panelNumber = new QLabel("0", NULL, "knmnumberdocknotifier");
  _panelNumber->setGeometry(0, 0, 32, 32);
  _panelNumber->setFont(QFont("Helvetica", 16, QFont::Bold));
  _panelNumber->setAlignment(Qt::AlignCenter);
  _panelNumber->show();

  QToolTip::add(_panelNumber, i18n("KNewMail NumberDock Notifier"));
  
  KWin::setSystemTrayWindowFor( _panelNumber->winId(), 0 );
  }
