/***************************************************************************
                knmsoundplayernotifier.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlabel.h>
#include <qfont.h>

#include <kconfig.h>
#include <kmainwindow.h>
#include <klocale.h>
#include <kprocess.h>

#include "knmsoundplayernotifier.h"
#include "soundplayerconfigdialog.h"

extern "C" {
  KNMPluginInfo *initSoundPlayerPluginInfo(QObject *parent)
    {
    return new KNMPluginInfo("SoundPlayer",
			     "Uses a selected application to play a selected sound file.",
			     "Mike Pilone <mpilone@slac.com>", "1.0.0",
			     parent, "knmsoundplayernotifierinfo");
    }
  
  KNMNotifier *initSoundPlayerPlugin(KMainWindow *parent)
    {
    return new KNMSoundPlayerNotifier(parent, "knmsoundplayernotifier");
    }
}

KNMSoundPlayerNotifier::KNMSoundPlayerNotifier(KMainWindow *parent,
					       const char *name)
  : KNMNotifier(initSoundPlayerPluginInfo(parent), parent, name),
    _filePath(QString::null), _playerPath(QString::null),
    _lastNumMessages(0)
  {
  }

KNMSoundPlayerNotifier::~KNMSoundPlayerNotifier()
  {
  }

void KNMSoundPlayerNotifier::readOptions(KConfig &config)
  {
  _playerPath = config.readEntry("Player Path", QString::null);
  _filePath = config.readEntry("File Path", QString::null);
  
  KNMNotifier::readOptions(config);
  }

void KNMSoundPlayerNotifier::writeOptions(KConfig &config)
  {
  config.writeEntry("Player Path", _playerPath);
  config.writeEntry("File Path", _filePath);
  
  KNMNotifier::writeOptions(config);
  }

void KNMSoundPlayerNotifier::updateState(QList<KNMMessage> &msgs)
  {
  // Run through the list of messages. If we have more messages than last
  // time, play a sound

  if ((msgs.count() > _lastNumMessages) && (_playerPath.length() > 0))
    {
    KProcess process;

    process << _playerPath << _filePath;
    process.start(KProcess::DontCare, KProcess::NoCommunication);
    }

  _lastNumMessages = msgs.count();
  }

void KNMSoundPlayerNotifier::updateMarkMessageForDelete(Q_UINT32)
  {
  // no-op
  }

void KNMSoundPlayerNotifier::updateUnmarkMessageForDelete(Q_UINT32)
  {
  // no-op
  }

bool KNMSoundPlayerNotifier::execConfigDialog(QWidget *parent,
					      const char *name)
  {
  QString pName = getPluginName();
  
  SoundPlayerConfigDialog dialog(pName, _filePath, _playerPath,
				 parent, name);

  if (dialog.exec())
    {
    _filePath = dialog.getFilePath();
    _playerPath = dialog.getPlayerPath();
    setPluginName(dialog.getPluginName());
    
    return true;
    }

  return false;
  }
