#include <qlabel.h>
#include <qframe.h>
#include <qlayout.h>
#include <qgroupbox.h>

#include <kurlrequester.h>
#include <klineedit.h>
#include <klocale.h>

#include "soundplayerconfigdialog.h"

SoundPlayerConfigDialog::SoundPlayerConfigDialog(const QString &pName,
						 const QString &filePath,
						 const QString &playerPath,
						 QWidget *parent,
						 const char *name)
  : KDialogBase(KDialogBase::Plain, i18n("Sound Player Config"),
		KDialogBase::Ok | KDialogBase::Cancel, KDialogBase::Ok,
		parent, name, true, true), _nameEdit(NULL),
    _fileRequester(NULL), _playerRequester(NULL)
  {
  QFrame *page = plainPage();
  QVBoxLayout *layout = new QVBoxLayout(page);
  layout->setSpacing(3);
  layout->setMargin(5);
  
  QGroupBox *gBox = new QGroupBox(2, Qt::Horizontal,
				  i18n("General"), page, "gBox");
  layout->addWidget(gBox);
  
  QLabel *label = new QLabel(i18n("Name:"), gBox);
  _nameEdit = new KLineEdit(pName, gBox, "nameEdit");

  gBox = new QGroupBox(2, Qt::Horizontal,
		       i18n("Sound"), page, "gBox");
  layout->addWidget(gBox);
  
  label = new QLabel(i18n("Player:"), gBox);
  _playerRequester = new KURLRequester(playerPath, gBox, "playerRequester",
				       true);

  label = new QLabel(i18n("Sound File:"), gBox);
  _fileRequester = new KURLRequester(filePath, gBox, "fileRequester",
				       true);
  }

SoundPlayerConfigDialog::~SoundPlayerConfigDialog()
  {
  }

QString SoundPlayerConfigDialog::getPluginName()
  {
  return _nameEdit->text();
  }

QString SoundPlayerConfigDialog::getFilePath()
  {
  return _fileRequester->lineEdit()->text();
  }

QString SoundPlayerConfigDialog::getPlayerPath()
  {
  return _playerRequester->lineEdit()->text();
  }
