/***************************************************************************
                     knmmessage.h  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KNMMESSAGE_H
#define KNMMESSAGE_H

#include <qobject.h>
#include <qstring.h>
#include <qdatetime.h>

#include <rmm/Envelope.h>

using namespace RMM;

/** Represents a mail message. Most of the functionality comes from the
 * Envelope class, which is part of librmm. See the librrm documentation
 * in kdepim for more information.
 */
class KNMMessage : public Envelope
  {
  public:
    KNMMessage(Q_UINT32 ownerUID, const QString &accountName,
	       const QString &msgHeader);
   
    virtual ~KNMMessage();

    /** @return the UID of the plugin that owns this message (the account)
     */
    Q_UINT32 getOwnerUID() const   { return _ownerUID; }

    /** @return The unique ID of this message. This is used for marking
     * delete in the notifiers.
     */
    Q_UINT32 getUniqueID() const   { return _uniqueID; }

    /** @return The account name the mesaage belongs to. This is dependant
     * apon the protocol that read the message, but it is usually something
     * like <username>@<server>
     */
  const QString &getAccountName() const { return _accountName; }

    /** Marks the message to be deleted.
     */
    void markForDelete() { _markedForDelete = true; }

    /** Unmarks the message for deletion
     */
    void unmarkForDelete() { _markedForDelete = false; }

    /** @return True if the message is marked for deletion.
     */
    bool isMarkedForDelete() const { return _markedForDelete; }

      private:
    Q_UINT32 _ownerUID;
    Q_UINT32 _uniqueID;
    bool _markedForDelete;

  QString _accountName;

  static Q_UINT32 _sUIDCounter;
  };

#endif
