/***************************************************************************
                         knmnotifier.h  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KNMNOTIFIER_H
#define KNMNOTIFIER_H

#include "knmplugin.h"
#include "knmmessage.h"

#include <qlist.h>

class KMainWindow;
class KConfig;

/** Base class for all Notifier Plugins.
 */
class KNMNotifier : public KNMPlugin
  {
  Q_OBJECT
  
    public:
  KNMNotifier(KNMPluginInfo *info,
	      KMainWindow *parent, const char *name);
  virtual ~KNMNotifier();

    virtual void readOptions(KConfig &c) { KNMPlugin::readOptions(c); }
    virtual void writeOptions(KConfig &c) { KNMPlugin::writeOptions(c); }

    /** Update the state of the plugin using the list of messages. Do not
     * alter the list! A message may already be marked for delete when this
     * method is called. If your plugin cares, you should check each message.
     */
  virtual void updateState(QList<KNMMessage> &msgs) = 0;

    /** Mark the given message for delete. This can be called any time
     * you have messages. Overload this method if you notifier cares
     * about messages being marked.
     */
    virtual void updateMarkMessageForDelete(Q_UINT32 msgID) { msgID = msgID; }

    /** Unmark the message for delete. This can be called any time
     * you have messages. Overload this method if you notifier cares
     * about messages being marked.
     */
    virtual void updateUnmarkMessageForDelete(Q_UINT32 msgID) {msgID = msgID;}

    /** If your notifier has a GUI component, initialize it here. This
     * method will be called when your notifier should become visible. Do
     * not show anything to the user until this method is called. When a
     * user is configuring your notifier, an instance will be created, but
     * this method will not be called, so the user will not see the fact that
     * the notifier exists until they confirm the config dialog.
     */
    virtual void initGUI() {}
    
  signals:
    /** Signals a force check of mail.
     */
  void forceCheck();

    /** Signals that the message should be marked for delete
     */
  void markMessageForDelete(Q_UINT32 msgID);

    
  /** Signals that the message should be unmarked for delete
   */
   void unmarkMessageForDelete(Q_UINT32 msgID);
  };

#endif
