/***************************************************************************
                         knmplugin.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qglobal.h>
#include <qlineedit.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qframe.h>

#include <kdebug.h>
#include <kconfig.h>
#include <klocale.h>

#include "knmplugin.h"

Q_UINT32 KNMPlugin::_sUIDCounter = 0;

////////////////////////////////////
// KNMPlugin

KNMPlugin::KNMPlugin(KNMPluginInfo *info,
		     QObject *parent, const char *name)
      : QObject(parent, name), _pluginInfo(info), _uid(_sUIDCounter++),
	_pluginName("Unknown")
  {
  if (_pluginInfo == NULL)
    qFatal("KNMPlugin::KNMPlugin: info parameter was NULL\n");
  }

KNMPlugin::~KNMPlugin()
  {
  }

void KNMPlugin::readOptions(KConfig &config)
  {
  _pluginName = config.readEntry("Name", "Unknown");
  }

void KNMPlugin::writeOptions(KConfig &config)
  {
  config.writeEntry("Name", _pluginName);
  }

bool KNMPlugin::execConfigDialog(QWidget *parent, const char *name)
  {
  KNMPluginConfigDialog dialog(getPluginName(), parent, name);
  if (dialog.exec() == QDialog::Accepted)
    {
      setPluginName(dialog.getPluginName());
      return true;
    }

  return false;
  }

///////////////////////////////////
// KNMPluginInfo

KNMPluginInfo::KNMPluginInfo(const QString &pluginType, const QString &desc,
			     const QString &author, const QString &version,
			     QObject *parent, const char *name)
  : QObject(parent, name),  _pluginType(pluginType), _description(desc),
    _author(author), _version(version)
  {
  }

KNMPluginInfo::~KNMPluginInfo()
  {
  }

///////////////////////////////////
// KNMPluginConfigDialog


KNMPluginConfigDialog::KNMPluginConfigDialog(const QString &pluginName,
					     QWidget *parent,
					     const char *name)
      : KDialogBase(KDialogBase::Plain, i18n("Plugin Configuration"),
		    KDialogBase::Ok | KDialogBase::Cancel, KDialogBase::Ok,
		    parent, name), _pluginNameEdit(NULL)
  {
  QLabel *label;
  
  QGroupBox *groupBox = new QGroupBox(2, Qt::Horizontal, i18n("General"),
			      plainPage(), "groupBox");

  label = new QLabel(i18n("Name:"), groupBox, "label");
  label->setMinimumSize(label->sizeHint());

  _pluginNameEdit = new QLineEdit(groupBox, "pluginNameEdit");
  _pluginNameEdit->setText(pluginName);
  _pluginNameEdit->setMinimumSize(_pluginNameEdit->sizeHint());

  groupBox->setMinimumSize(groupBox->sizeHint());

  plainPage()->setMinimumSize(groupBox->sizeHint());
  }

KNMPluginConfigDialog::~KNMPluginConfigDialog()
  {
  }

QString KNMPluginConfigDialog::getPluginName()
  {
  return _pluginNameEdit->text();
  }
