/***************************************************************************
                         knmplugin.h  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KNMPLUGIN_H
#define KNMPLUGIN_H

#include <qobject.h>
#include <qwidget.h>

#include <kdialogbase.h>

class QLineEdit;
class KConfig;
class KNMPluginInfo;

/** The base class of all KNewMail plugins (Notifiers, Protocols,
 * Filters, etc).
 */
class KNMPlugin : public QObject
  {
  Q_OBJECT
  
  public:
  /** Constructor. Takes an info class so it can return information about
   * itself when requested.
   */
  KNMPlugin(KNMPluginInfo *info,
	    QObject *parent, const char *name);
    virtual ~KNMPlugin();
    
    /** @return A reference to the plugin info object.
     */
    const KNMPluginInfo &getPluginInfo() { return *_pluginInfo; }

    /** @return The unique identifier for this plugin. The ID is an
     * integer and is not persistant. It will be generated when the
     * plugin is instantiated.
     */
  Q_UINT32 getUniqueID()            { return _uid; }

    /** @return The name of the plugin. This is the user defined name,
     * not the actual name of the plugin, that can be obtained
     * from the info.
     */
    const QString &getPluginName() const { return _pluginName; }
    void setPluginName(const QString &name) { _pluginName = name; }

    /** Should be overloaded in subclasses to read any and all information
     * related to this plugin. Do not change the group of the config file,
     * it will be preset for you. Be sure to call the superclasses
     * readOptions method from yours.
     */
  virtual void readOptions(KConfig &);

    /** @see readOptions
     */
  virtual void writeOptions(KConfig &);

    /** Used to display the config dialog of the plugin. By default,
     * all plugins have a configurable name. If you want more options
     * in your plugin, overload this method. Do not call the super classes
     * method if you overload this, or the user will see two config
     * dialogs! Be sure your config dialog allows the user to configure
     * everything in the super class as well (i.e. plugin name).
     *
     * @return True if the config changed, false otherwise.
     */
    virtual bool execConfigDialog(QWidget *parent, const char *name);
    
    private:
  static Q_UINT32 _sUIDCounter;

  KNMPluginInfo *_pluginInfo;
  Q_UINT32 _uid;

    QString _pluginName;
  };

/** Contains all the information about a plugin.
 */
class KNMPluginInfo : public QObject
  {
    public:
    /** Constructor.
     *
     * @param pluginType The type of plugin (POP3, NumberDock, etc)
     * @param desc A description of the plugin. This should be limited to
     * one or two lines only.
     * @param author The author of the plugin
     * @param version The version of the plugin
     */
  KNMPluginInfo(const QString &pluginType, const QString &desc,
		const QString &author, const QString &version,
		QObject *parent, const char *name);
  ~KNMPluginInfo();
  
  const QString &getPluginType()  const { return _pluginType; }
  const QString &getDescription() const { return _description; }
  const QString &getAuthor() const      { return _author; }
  const QString &getVersion() const     { return _version; }

    private:
  QString _pluginType;
  QString _description;
  QString _author;
  QString _version;
  };

/** Basic config dialog. This dialog is used if the user doesn't overload
 * execConfigDialog method.
 */
class KNMPluginConfigDialog : public KDialogBase
  {
      public:
    KNMPluginConfigDialog(const QString &pluginName, QWidget *parent,
			  const char *name);
    ~KNMPluginConfigDialog();

    QString getPluginName();

      private:
    QLineEdit *_pluginNameEdit;
  };

#endif
