/***************************************************************************
                         knmprotocol.h  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KNMPROTOCOL_H
#define KNMPROTOCOL_H

#include "knmplugin.h"
#include "knmmessage.h"

#include <qlist.h>

class KConfig;

/** The base class of all KNewMail protocols. The naming here is a little
 * weird. A protocol implements a way to talk to a certain server
 * (i.e. POP3, IMAP4), however when a protocol is instantiate, it becomes
 * an account. The user may have multiple accounts (multiple protocols or
 * multiple instances of the same protocol).
 */
class KNMProtocol : public KNMPlugin
  {
  Q_OBJECT
  
    public:
  KNMProtocol(KNMPluginInfo *info,
	      QObject *parent, const char *name);
  virtual ~KNMProtocol();

    /** Gives a list of messages to delete. If the protocol cannot identify
     * the message using the KNMMessage class, you should inherit from
     * the message class and implement methods to carry your needed data.
     */
  virtual void deleteMessages(QList<KNMMessage> &msgs);

    /** Check for new messages.
     */
  virtual void checkMessages();

    /** Cancel the current check. This may be called while the protocol is
     * in the delete stage.
     */
  virtual void cancelCheck() = 0;

    /** @return The last error the account encountered.
     */
    virtual QString getLastError() = 0;

    /** @return The account name. This usually something like
     * <username>@<server>
     */
    virtual QString getAccountName() = 0;

    virtual void readOptions(KConfig &c) { KNMPlugin::readOptions(c); }
    virtual void writeOptions(KConfig &c) {KNMPlugin::writeOptions(c); }

    /** @return True if currently checking
     */
  bool isChecking() { return _checking; }

    /** @return True if currently deleting
     */
  bool isDeleting() { return _deleting; }

  signals:
  /** Signals that a message was recieved. The message will be
   * stored, so the pointer should not be deleted.
   */
  void messageReceived(KNMMessage *msg);

    /** Signals that the protocol is done checking.
     */
  void doneChecking(bool success, Q_UINT32);

    /** Signals that the protocol is done deleting.
     */
  void doneDeleting(bool success, Q_UINT32);

 protected slots:
 virtual void slotDoneChecking(bool, Q_UINT32);
  virtual void slotDoneDeleting(bool, Q_UINT32);
  
    private:
  bool _checking;
  bool _deleting;
  };

#endif
