#include <qlineedit.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qgroupbox.h>

#include <klocale.h>

#include "imapconfigdialog.h"

ImapConfigDialog::ImapConfigDialog(const QString &pluginName,
				   const QString &login,
				   const QString &password,
				   const QString &host,
				   const QString &mailboxName,
				   Q_UINT32 port,
				   bool storePassword,
				   QWidget *parent, const char *name)
  : KDialogBase(KDialogBase::Plain, i18n("IMAP3 Configuration"),
		KDialogBase::Ok | KDialogBase::Cancel,
		KDialogBase::Ok, parent, name, true, true),
    _pluginNameEdit(NULL), _loginEdit(NULL), _mailboxNameEdit(NULL),
    _passwordEdit(NULL), _hostEdit(NULL), _portSpinBox(NULL),
    _storePassword(NULL)
  {
  QLabel *label = NULL;
  QGroupBox *box = NULL;
  QFrame *page = plainPage();
  QVBoxLayout *layout = new QVBoxLayout(page);
  layout->setSpacing(3);
  layout->setMargin(2);

  box = new QGroupBox(2, Qt::Horizontal, i18n("General"),
		      page, "box");
  layout->addWidget(box);
  
  label = new QLabel(i18n("Name:"), box);
  
  _pluginNameEdit = new QLineEdit(box, "pluginNameEdit");
  _pluginNameEdit->setText(pluginName);

  box = new QGroupBox(2, Qt::Horizontal, i18n("Identity"),
		      page, "box");
  layout->addWidget(box);
  
  label = new QLabel(i18n("Login:"), box);

  _loginEdit = new QLineEdit(box, "loginEdit");
  _loginEdit->setText(login);

  label = new QLabel(i18n("Password"), box);

  _passwordEdit = new QLineEdit(box, "passwordEdit");
  _passwordEdit->setEchoMode(QLineEdit::Password);
  _passwordEdit->setText(password);

  _storePassword = new QCheckBox(box, "storePassword");
  _storePassword->setChecked(storePassword);

  label = new QLabel(i18n("Store password in config file"),
		     box);
  
  box = new QGroupBox(2, Qt::Horizontal, i18n("Server"),
		      page, "box");
  layout->addWidget(box);
  
  label = new QLabel(i18n("Host:"), box);

  _hostEdit = new QLineEdit(box, "hostEdit");
  _hostEdit->setText(host);

  label = new QLabel(i18n("Port:"), box);

  _portSpinBox = new QSpinBox(1, 65000, 1, box, "portSpinBox");
  _portSpinBox->setValue(port);

  label = new QLabel(i18n("Folder:"), box);

  _mailboxNameEdit = new QLineEdit(box, "mailboxNameEdit");
  _mailboxNameEdit->setText(mailboxName);
  }

ImapConfigDialog::~ImapConfigDialog()
  {
  }

QString ImapConfigDialog::getPluginName() const
  {
  return _pluginNameEdit->text();
  }

QString ImapConfigDialog::getLogin() const
  {
  return _loginEdit->text();
  }

QString ImapConfigDialog::getPassword() const
  {
  return _passwordEdit->text();
  }

QString ImapConfigDialog::getHost() const
  {
  return _hostEdit->text();
  }

Q_UINT32 ImapConfigDialog::getPort() const
  {
  return _portSpinBox->value();
  }

bool ImapConfigDialog::shouldStorePassword()
  {
  return _storePassword->isChecked();
  }

QString ImapConfigDialog::getMailboxName() const
  {
  return _mailboxNameEdit->text();
  }
