/***************************************************************************
                     knmimap4protocol.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string.h>

#include <kdebug.h>
#include <kconfig.h>
#include <kio/global.h>

#include "knmimap4protocol.h"
#include "imapconfigdialog.h"

using namespace KIO;

extern "C" {
  KNMPluginInfo *initIMAP4PluginInfo(QObject *parent)
    {
    return new KNMPluginInfo("IMAP4", "IMAP4 Implementation (Read Only)",
			     "Mike Pilone <mpilone@slac.com>", "1.0.0",
			     parent, "knmimap4protocolinfo");
    }
  
  KNMProtocol *initIMAP4Plugin(QObject *parent)
    {
    return new KNMImap4Protocol(parent, "knmprotocolimap4");
    }
}

KNMImap4Protocol::KNMImap4Protocol(QObject *parent, const char *name)
  : KNMProtocol(initIMAP4PluginInfo(parent),
		parent, name), _currentImapID(0), _buffer(),
    _state(Idle), _lastError(""), _numEntryLists(0),
    _fetchMessages(), _deadMessages(),
    _canceling(false), _login(""), _password(""), _host(""), _mailboxName(""),
    _port(143), _storePassword(true)
  {
  }

KNMImap4Protocol::~KNMImap4Protocol()
  {
  }

void KNMImap4Protocol::readOptions(KConfig &config)
  {
  KNMProtocol::readOptions(config);
  
  // Read our own options here
  _login = config.readEntry("Login", "");
  _storePassword = config.readBoolEntry("StorePassword", true);
  if (_storePassword)
    _password = config.readEntry("Password", "");
  _host = config.readEntry("Host", "");
  _port = config.readNumEntry("Port", 110);
  _mailboxName = config.readEntry("Mailbox Name", "INBOX");
  }

void KNMImap4Protocol::writeOptions(KConfig &config)
  {
  KNMProtocol::writeOptions(config);

  // Write our own options here
  config.writeEntry("Login", _login);
  config.writeEntry("StorePassword", _storePassword);
  if (_storePassword)
    config.writeEntry("Password", _password);
  config.writeEntry("Host", _host);
  config.writeEntry("Port", _port);
  config.writeEntry("Mailbox Name", _mailboxName);
  }
  
void KNMImap4Protocol::deleteMessages(QList<KNMMessage> &msgs)
  {
  KNMProtocol::deleteMessages(msgs);
  
  // Currently we don't know how to delete!
  emit doneDeleting(true, getUniqueID());
  
  
//   _state = Delete;
//   _canceling = false;
  
//   QListIterator<KNMMessage> iter(msgs);
//   for (iter.toFirst(); iter.current(); ++iter)
//     {
//     _deadMessages << ((KNMImap4Message*)*iter)->getImapID();
//     }    

//   deleteSingleMessage();
  }

void KNMImap4Protocol::checkMessages()
  {
  KNMProtocol::checkMessages();
  
  _canceling = false;
  buildMessageList();
  }

void KNMImap4Protocol::cancelCheck()
  {
  _canceling = true;
  }
  
QString KNMImap4Protocol::getLastError()
  {
  return _lastError;
  }

QString KNMImap4Protocol::getAccountName()
  {
  QString accountName = _login + "@";
  accountName += _host;
  
  return accountName;
  }

void KNMImap4Protocol::buildMessageList()
  {
  //kdDebug() << "KNMImap4Protocol::getNumMessages\n";
  _state = CheckList;
  _buffer = "";
  _numEntryLists = 0;
  QString url = buildURL() + "/";
  url += _mailboxName + "/";
  
  KIO::SimpleJob *job = KIO::listDir(url,
			   false);

  connect(job, SIGNAL(entries(KIO::Job *, const KIO::UDSEntryList&)),
	  this, SLOT(slotEntries(KIO::Job *, const KIO::UDSEntryList&)));
  connect(job, SIGNAL(result(KIO::Job *)), this, SLOT(slotResult(KIO::Job *)));
  }

void KNMImap4Protocol::getMessage(Q_UINT32 msgNum)
  {
  //kdDebug() << "KNMImap4Protocol::getMessage: " << msgNum << "\n";

  _buffer = "";
  QString url = buildURL() + "/";
  url += _mailboxName + "/;UID=";
  url += QString::number(msgNum);
  url += "/;section=HEADER";
  
  KIO::SimpleJob *job = KIO::get(url, false, false);
 
  connect(job, SIGNAL(result(KIO::Job *)), this, SLOT(slotResult(KIO::Job *)));
  connect(job, SIGNAL(data(KIO::Job *, const QByteArray &)),
	  this, SLOT(slotData(KIO::Job *, const QByteArray &)));
  }

void KNMImap4Protocol::deleteSingleMessage()
  {
  // Not implemented for IMAP

  
  // if we have anything in the list, start deleting
//   if (_deadMessages.count() > 0)
//     {
//     Q_UINT32 id = *(_deadMessages.begin());
//     _deadMessages.remove(_deadMessages.begin());

//     QString url = buildURL() + "remove/";
//     url += QString::number(id);
    
//     //QString fakeUrl = "imap4://mpilone:pebrim1@scooby.stl.vt.edu/index"; 
//     //kdDebug() << "Would Run: " << url << "\n"
//     //      << "Real Run : " << fakeUrl << "\n";
    
//     KIO::Job *job = KIO::get(url, false, false);

//     connect(job, SIGNAL(result(KIO::Job *)),
// 	    this, SLOT(slotResult(KIO::Job *)));
//     connect(job, SIGNAL(data(KIO::Job *, const QByteArray &)),
// 	  this, SLOT(slotData(KIO::Job *, const QByteArray &)));
//     }
//   else if (_state == DeleteClose)
//     {
//     _state = Idle;
//     emit doneDeleting(true, getUniqueID());
//     }
//   else
//     {
//     // Need to send close command
//     QString url = buildURL() + "commit";
//     KIO::Job *job = KIO::get(url, false, false);

//     _state = DeleteClose;
    
//     connect(job, SIGNAL(result(KIO::Job *)),
// 	    this, SLOT(slotResult(KIO::Job *)));
//     connect(job, SIGNAL(data(KIO::Job *, const QByteArray &)),
// 	  this, SLOT(slotData(KIO::Job *, const QByteArray &)));
//     }
  }

bool KNMImap4Protocol::execConfigDialog(QWidget *parent, const char *name)
  {
  bool retVal = false;
  ImapConfigDialog dialog(getPluginName(), _login, _password, _host,
			 _mailboxName, _port, _storePassword, parent, name);

  if (dialog.exec())
    {
    // save the values
    setPluginName(dialog.getPluginName());
    _login = dialog.getLogin();
    _password = dialog.getPassword();
    _host = dialog.getHost();
    _port = dialog.getPort();
    _storePassword = dialog.shouldStorePassword();
    _mailboxName = dialog.getMailboxName();
    
    retVal = true;
    }

  return retVal;
  }

QString KNMImap4Protocol::buildURL()
  {
  QString url = "imap://";
  url += _login;

  if (_password.length() > 0)
    {
    url += ":";
    url += _password;
    }

  url += "@";
  url += _host;
  url += ":";
  url += QString::number(_port) ;

  return url;
  }

//////////////////////////////
// SLOTS
void KNMImap4Protocol::slotResult(KIO::Job *job)
  {
  //kdDebug() << "KNMImap4Protocol::slotResult\n"
  //          << &(*job) << "\n";
  
  // Check if it was an error
  if (job && (job->error() != 0))
    {
      _lastError = job->errorString();
      //kdDebug() << "Error code: " << job->error() << "\n";
      _state = Idle;
      
      if (_state == Delete)
	emit doneDeleting(false, getUniqueID());
      else
	emit doneChecking(false, getUniqueID());
    }
  else if (_canceling)
    {
    _state = Idle;
    if ((_state == Delete) || (_state == DeleteClose))
      emit doneDeleting(true, getUniqueID());
    else
      emit doneChecking(true, getUniqueID());
    }
  else
    {
    //kdDebug() << "---------------\n"
    //     << "Final Buffer:\n" << _buffer << "\n"
    //    << "---------------\n";
    
    switch (_state)
      {
      case Idle:
	  qWarning("KNMImap4Protocol::slotResult: got a result while idle");
	  break;

      case CheckList:
	  //kdDebug() << "KNMImap4Protocol::slotResult: "
	  //	<< _numMessages << "\n";
	  if (_fetchMessages.count() > 0)
	    {
	    _state = CheckFetch;
	    _currentImapID = _fetchMessages[0];
	    _fetchMessages.remove(_fetchMessages.begin());
	    getMessage(_currentImapID);
	    }
	  else
	    {
	    // we must be done
	    _state = Idle;
	    emit doneChecking(true, getUniqueID());
	    }
	  break;

      case CheckFetch:
      {
      KNMMessage *msg = new KNMImap4Message(_currentImapID,
					   getUniqueID(), getAccountName(),
					   _buffer);
      emit messageReceived(msg);
      if (_fetchMessages.count() > 0)
	{
	_buffer = "";
	_currentImapID = _fetchMessages[0];
	_fetchMessages.remove(_fetchMessages.begin());
	getMessage(_currentImapID);
	}
      else
	{
	// we must be done
	_state = Idle;
	emit doneChecking(true, getUniqueID());
	}
      break;
      }

      case DeleteClose:
      case Delete:
	  // Keep deleting.
	  deleteSingleMessage();
	  break;	    
      }
    }
  }

void KNMImap4Protocol::slotData(KIO::Job *, const QByteArray &data)
  {
  //kdDebug() << data.data() << "\n";

 if (data.size() == 0)
    return;

 // Make a copy so we can modify it
  QByteArray dataCopy(data.size() + 1);

  memcpy(dataCopy.data(), data.data(), data.size());
  dataCopy.data()[dataCopy.size()-1] = '\0';

  //kdDebug() << dataCopy.data() << "\n";
 
  _buffer += "\n"; 
  _buffer.append(QString(dataCopy.data()));
  }


void KNMImap4Protocol::slotEntries(KIO::Job *, const KIO::UDSEntryList &list)
  {
  UDSEntryList::ConstIterator iter;
  UDSEntry::ConstIterator iter2;
  UDSAtom atom;
  
  _numEntryLists++;

  kdDebug() << "Count: " << list.count() << "\n";
  
  // We only care if this is the second listing. There isn't much
  // documentation on the IMAP slave, but it seems to be that the
  // second listing is the UNSEEN messages and the last listing are all
  // messages. I'm not sure what the first is :(
  //
  // Ugg. Ok, so the second listing seems to be the correct number of
  // recent (UNSEEN) messages, but the ID is not correct. I'll just have
  // to get all the messages.
  for (iter = list.begin(); (iter != list.end()); ++iter)
    {
    UDSEntry entry = *iter;

    // We only need the first atom, the message ID
    iter2 = entry.begin();
    atom = *iter2;

    _fetchMessages << atom.m_str.toUInt();
    }

  kdDebug() << "--- Done Listing Entries\n";
  }
