/***************************************************************************
                     knmimap4protocol.h  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KNM_IMAP4_PROTOCOL_H
#define KNM_IMAP4_PROTOCOL_H

#include <qstring.h>
#include <qarray.h>
#include <qvaluelist.h>

#include <kio/job.h>

#include "knmprotocol.h"
#include "knmmessage.h"

class KConfig;

class KNMImap4Protocol : public KNMProtocol
  {
    Q_OBJECT
    
  public:
    KNMImap4Protocol(QObject *parent, const char *name);
    ~KNMImap4Protocol();
    
    virtual void readOptions(KConfig &);
    virtual void writeOptions(KConfig &);
    
    virtual void deleteMessages(QList<KNMMessage> &msgs);
    virtual void checkMessages();
    virtual void cancelCheck();
    
    virtual QString getLastError();
    virtual QString getAccountName();
    
    virtual bool execConfigDialog(QWidget *parent, const char *name);
    
  protected slots:
    void slotResult(KIO::Job *);
    void slotData(KIO::Job *, const QByteArray &);
    void slotEntries(KIO::Job *, const KIO::UDSEntryList &);
    
  private:
    void buildMessageList();
    void getMessage(Q_UINT32 msgNum);
    void deleteSingleMessage();
    QString buildURL();
    
    enum ProtocolState { Idle, Delete, DeleteClose, CheckList, CheckFetch };
    
    Q_UINT32 _currentImapID;
    QString _buffer;
    ProtocolState _state;
    QString _lastError;
    Q_UINT32 _numEntryLists;
    
    QValueList<Q_UINT32> _fetchMessages;
    QValueList<Q_UINT32> _deadMessages;

  bool _canceling;

  QString _login;
  QString _password;
  QString _host;
    QString _mailboxName;
  Q_UINT32 _port;
  bool _storePassword;
  };

class KNMImap4Message : public KNMMessage
  {
    public:
  KNMImap4Message(Q_UINT32 imapID,
		 Q_UINT32 ownerUID, const QString &accountName,
		 const QString &rawHeader)
    : KNMMessage(ownerUID, accountName, rawHeader),
      _imapID(imapID)
    {
    }

  Q_UINT32 getImapID() const { return _imapID; }
  
    private:
  Q_UINT32 _imapID;
  };

#endif
