/***************************************************************************
                     knmprotocolpop3.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string.h>

#include <kdebug.h>
#include <kconfig.h>

#include "knmpop3protocol.h"
#include "popconfigdialog.h"

extern "C" {
  KNMPluginInfo *initPOP3PluginInfo(QObject *parent)
    {
    return new KNMPluginInfo("POP3", "Post Office Protocol 3 Implementation",
			     "Mike Pilone <mpilone@slac.com>", "1.0.0",
			     parent, "knmpop3protocolinfo");
    }
  
  KNMProtocol *initPOP3Plugin(QObject *parent)
    {
    return new KNMPop3Protocol(parent, "knmprotocolpop3");
    }
}

KNMPop3Protocol::KNMPop3Protocol(QObject *parent, const char *name)
  : KNMProtocol(initPOP3PluginInfo(parent),
		parent, name), _numMessages(0), _buffer(),
    _state(Idle), _lastError(""), _deadMessages(), _canceling(false),
    _login(""), _password(""), _host(""), _port(110), _storePassword(true)
  {
  }

KNMPop3Protocol::~KNMPop3Protocol()
  {
  }

void KNMPop3Protocol::readOptions(KConfig &config)
  {
  KNMProtocol::readOptions(config);

  // Read our own options here
  _login = config.readEntry("Login", "");
  _storePassword = config.readBoolEntry("StorePassword", true);
  if (_storePassword)
    _password = config.readEntry("Password", "");
  _host = config.readEntry("Host", "");
  _port = config.readNumEntry("Port", 110);
  }

void KNMPop3Protocol::writeOptions(KConfig &config)
  {
  KNMProtocol::writeOptions(config);

  // Write our own options here
  config.writeEntry("Login", _login);
  config.writeEntry("StorePassword", _storePassword);
  if (_storePassword)
    config.writeEntry("Password", _password);
  config.writeEntry("Host", _host);
  config.writeEntry("Port", _port);
  }
  
void KNMPop3Protocol::deleteMessages(QList<KNMMessage> &msgs)
  {
  KNMProtocol::deleteMessages(msgs);
  
  _state = Delete;
  _canceling = false;
  
  QListIterator<KNMMessage> iter(msgs);
  for (iter.toFirst(); iter.current(); ++iter)
    {
    _deadMessages << ((KNMPop3Message*)*iter)->getPopID();
    }    

  deleteSingleMessage();
  }

void KNMPop3Protocol::checkMessages()
  {
  KNMProtocol::checkMessages();
  
  _canceling = false;
  getNumMessages();
  }

void KNMPop3Protocol::cancelCheck()
  {
  _canceling = true;
  }
  
QString KNMPop3Protocol::getLastError()
  {
  return _lastError;
  }

QString KNMPop3Protocol::getAccountName()
  {
  QString accountName = _login + "@";
  accountName += _host;
  
  return accountName;
  }

void KNMPop3Protocol::getNumMessages()
  {
  //kdDebug() << "KNMPop3Protocol::getNumMessages\n";
  _state = CheckList;
  _buffer = "";
  
  KIO::Job *job = KIO::get(buildURL() + "index",
			   false, false);
  
  connect(job, SIGNAL(result(KIO::Job *)), this, SLOT(slotResult(KIO::Job *)));
  connect(job, SIGNAL(data(KIO::Job *, const QByteArray &)),
	  this, SLOT(slotData(KIO::Job *, const QByteArray &)));
  }

void KNMPop3Protocol::getMessage(Q_UINT32 msgNum)
  {
  //kdDebug() << "KNMPop3Protocol::getMessage: " << msgNum << "\n";

  _buffer = "";
  QString url = buildURL() + "headers/";
  url += QString::number(msgNum);
  
  KIO::Job *job = KIO::get(url, false, false);
  
  connect(job, SIGNAL(result(KIO::Job *)), this, SLOT(slotResult(KIO::Job *)));
  connect(job, SIGNAL(data(KIO::Job *, const QByteArray &)),
	  this, SLOT(slotData(KIO::Job *, const QByteArray &)));
  }

Q_UINT32 KNMPop3Protocol::countLines(const QString &buf)
  {
  Q_UINT32 numLines = 0;
  Q_INT32 index = -1;
  
  while ((index = buf.find('\n', index+1)) != -1)
    {
    numLines++;
    }

  return numLines;
  }

void KNMPop3Protocol::deleteSingleMessage()
  {
  // if we have anything in the list, start deleting
  if (_deadMessages.count() > 0)
    {
    Q_UINT32 id = *(_deadMessages.begin());
    _deadMessages.remove(_deadMessages.begin());

    QString url = buildURL() + "remove/";
    url += QString::number(id);
    
    //QString fakeUrl = "pop3://mpilone:pebrim1@scooby.stl.vt.edu/index"; 
    //kdDebug() << "Would Run: " << url << "\n"
    //      << "Real Run : " << fakeUrl << "\n";
    
    KIO::Job *job = KIO::get(url, false, false);

    connect(job, SIGNAL(result(KIO::Job *)),
	    this, SLOT(slotResult(KIO::Job *)));
    connect(job, SIGNAL(data(KIO::Job *, const QByteArray &)),
	  this, SLOT(slotData(KIO::Job *, const QByteArray &)));
    }
  else if (_state == DeleteClose)
    {
    _state = Idle;
    emit doneDeleting(true, getUniqueID());
    }
  else
    {
    // Need to send close command
    QString url = buildURL() + "commit";
    KIO::Job *job = KIO::get(url, false, false);

    _state = DeleteClose;
    
    connect(job, SIGNAL(result(KIO::Job *)),
	    this, SLOT(slotResult(KIO::Job *)));
    connect(job, SIGNAL(data(KIO::Job *, const QByteArray &)),
	  this, SLOT(slotData(KIO::Job *, const QByteArray &)));
    }
  }

bool KNMPop3Protocol::execConfigDialog(QWidget *parent, const char *name)
  {
  bool retVal = false;
  PopConfigDialog dialog(getPluginName(), _login, _password, _host,
			 _port, _storePassword, parent, name);

  if (dialog.exec())
    {
    // save the values
    setPluginName(dialog.getPluginName());
    _login = dialog.getLogin();
    _password = dialog.getPassword();
    _host = dialog.getHost();
    _port = dialog.getPort();
    _storePassword = dialog.shouldStorePassword();
    
    retVal = true;
    }

  return retVal;
  }

QString KNMPop3Protocol::buildURL()
  {
  QString url = "pop3://";
  url += _login;

  if (_password.length() > 0)
    {
    url += ":";
    url += _password;
    }

  url += "@";
  url += _host;
  url += ":";
  url += QString::number(_port) + "/";

  return url;
  }

//////////////////////////////
// SLOTS
void KNMPop3Protocol::slotResult(KIO::Job *job)
  {
  //kdDebug() << "KNMPop3Protocol::slotResult\n"
  //          << &(*job) << "\n";
  
  // Check if it was an error
  if (job && (job->error() != 0))
    {
      _lastError = job->errorString();
      //kdDebug() << "Error code: " << job->error() << "\n";
      _state = Idle;
      
      if (_state == Delete)
	emit doneDeleting(false, getUniqueID());
      else
	emit doneChecking(false, getUniqueID());
    }
  else if (_canceling)
    {
    _state = Idle;
    if ((_state == Delete) || (_state == DeleteClose))
      emit doneDeleting(true, getUniqueID());
    else
      emit doneChecking(true, getUniqueID());
    }
  else
    {
    //kdDebug() << "---------------\n"
    //      << "Final Buffer:\n" << _buffer << "\n"
    //      << "---------------\n";
    
    switch (_state)
      {
      case Idle:
	  qWarning("KNMPop3Protocol::slotResult: got a result while idle");
	  break;

      case CheckList:
	  _numMessages = countLines(_buffer);
	  //kdDebug() << "KNMPop3Protocol::slotResult: "
	  //	<< _numMessages << "\n";
	  if (_numMessages > 0)
	    {
	    _state = CheckFetch;
	    _buffer = "";
	    getMessage(_numMessages);
	    _numMessages--;	
	    }
	  else
	    {
	    // we must be done
	    _state = Idle;
	    emit doneChecking(true, getUniqueID());
	    }
	  break;

      case CheckFetch:
      {
      KNMMessage *msg = new KNMPop3Message(_numMessages+1,
					   getUniqueID(), getAccountName(),
					   _buffer);
      emit messageReceived(msg);
      if (_numMessages > 0)
	{
	_buffer = "";
	getMessage(_numMessages);
	_numMessages--;
	}
      else
	{
	// we must be done
	_state = Idle;
	emit doneChecking(true, getUniqueID());
	}
      break;
      }

      case DeleteClose:
      case Delete:
	  // Keep deleting.
	  deleteSingleMessage();
	  break;	    
      }
    }
  }

void KNMPop3Protocol::slotData(KIO::Job *, const QByteArray &data)
  {
  //kdDebug() << "KNMPop3Protocol::slotData\n";

 if (data.size() == 0)
    return;

 // Make a copy so we can modify it
  QByteArray dataCopy(data.size() + 1);

  memcpy(dataCopy.data(), data.data(), data.size());
  dataCopy.data()[dataCopy.size()-1] = '\0';

  //kdDebug() << dataCopy.data() << "\n";
 
  _buffer += "\n"; 
  _buffer.append(QString(dataCopy.data()));
  }


