/***************************************************************************
                     knmprotocolpop3.h  -  description
                             -------------------
    begin                : Sat Apr 28 19:10:24 EDT 2001
    copyright            : (C) 2001 by Mike Pilone
    email                : mpilone@slac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KNM_POP3_PROTOCOL_H
#define KNM_POP3_PROTOCOL_H

#include <qstring.h>
#include <qarray.h>
#include <qvaluelist.h>

#include <kio/job.h>

#include "knmprotocol.h"
#include "knmmessage.h"

class KConfig;

class KNMPop3Protocol : public KNMProtocol
  {
  Q_OBJECT
  
    public:
  KNMPop3Protocol(QObject *parent, const char *name);
  ~KNMPop3Protocol();
  
  virtual void readOptions(KConfig &);
  virtual void writeOptions(KConfig &);
  
  virtual void deleteMessages(QList<KNMMessage> &msgs);
  virtual void checkMessages();
  virtual void cancelCheck();
  
  virtual QString getLastError();
  virtual QString getAccountName();
    
  virtual bool execConfigDialog(QWidget *parent, const char *name);

 protected slots:
 void slotResult(KIO::Job *);
  void slotData(KIO::Job *, const QByteArray &);

    private:
    void getNumMessages();
    void getMessage(Q_UINT32 msgNum);
  Q_UINT32 countLines(const QString &);
  void deleteSingleMessage();
  QString buildURL();
  
  enum ProtocolState { Idle, Delete, DeleteClose, CheckList, CheckFetch };
  
  Q_UINT32 _numMessages;
  QString _buffer;
  ProtocolState _state;
  QString _lastError;

  QValueList<Q_UINT32> _deadMessages;

  bool _canceling;

  QString _login;
  QString _password;
  QString _host;
  Q_UINT32 _port;
  bool _storePassword;
  };

class KNMPop3Message : public KNMMessage
  {
    public:
  KNMPop3Message(Q_UINT32 popID,
		 Q_UINT32 ownerUID, const QString &accountName,
		 const QString &rawHeader)
    : KNMMessage(ownerUID, accountName, rawHeader),
      _popID(popID)
    {
    }

  Q_UINT32 getPopID() const { return _popID; }
  
    private:
  Q_UINT32 _popID;
  };

#endif
